interface ICollection<T> extends Iterable<T> {
    /**
     * Count the element in the collection.
     * @getter
     */
    readonly size: number;
    /**
     * Release memory.
     */
    destroy(): void;
    /**
     * Remove all elements.
     * Notice that this method does not release memory.
     */
    clear(): void;
}

/**
 * Adjacent list
 */
declare type IAdjacentList = number[][];
interface IDigraphEdge {
    /**
     * Target point of the edge.
     */
    to: number;
}
interface IDigraph<E extends IDigraphEdge = IDigraphEdge> {
    /**
     * The number of nodes in the graph. (0-index)
     */
    N: number;
    /**
     * Adjacency list. G[i] represent the index list of the edges start from node i.
     */
    G: number[][];
    /**
     *
     */
    edges: E[];
}

declare type ICompare<T> = (x: T, y: T) => -1 | 0 | 1 | number;
declare type IEquals<T> = (x: T, y: T) => boolean;
declare type IKeyOf<T, K = number> = (element: T) => K;
/**
 * Make all properties in `T` mutable.
 * @see https://stackoverflow.com/a/46634877
 */
declare type Mutable<T extends object> = {
    -readonly [P in keyof T]: T[P];
};
/**
 * Make a set of properties by key `K` become optional from `T`.
 */
declare type PickPartial<T extends object, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
/**
 * Make an object deep-readonly.
 */
declare type DeepReadonly<T extends object> = {
    readonly [P in keyof T]: T[P] extends object ? DeepReadonly<T[P]> : Readonly<T[P]>;
};
/**
 * Get element type of an array
 */
declare type ElementOfArray<T extends ReadonlyArray<unknown> | string> = T extends ReadonlyArray<infer ElementType> ? ElementType : T extends string ? string : never;

interface IOperator<T> {
    readonly ZERO: T;
    readonly ONE: T;
    add(x: T, y: T): T;
    subtract(x: T, y: T): T;
    divide(x: T, y: T): T;
    multiple(x: T, y: T): T;
}

export { DeepReadonly, ElementOfArray, IAdjacentList, ICollection, ICompare, IDigraph, IDigraphEdge, IEquals, IKeyOf, IOperator, Mutable, PickPartial };
